#include "asterx1_process.h"
#include "Epoch.hpp"
#include "UTCTime.hpp"
#include "TimeSystem.hpp"

CAsteRx1Process::CAsteRx1Process()
{
  unsigned int i=0;

  this->ephemeris_data.resize(37);// set the ephemeris vector for each of the satellites
  for(i=0;i<37;i++)
  {
    this->ephemeris_data[i].sync=false;
    this->ephemeris_data[i].missing_frames.resize(3,true);
  }
}

void CAsteRx1Process::generate_gps_nav(gpstk::EngEphemeris &ephemeris,TGPSNav &gps_nav)
{
  gpstk::CommonTime time;

  try{
    time=ephemeris.getTransmitTime();
    time.setTimeSystem(gpstk::TimeSystem(gpstk::TimeSystem::GPS));
    gpstk::Epoch gps_epoch(time);
    gps_nav.gps_timestamp.tow=gps_epoch.GPSsow()*1000;
    gps_nav.gps_timestamp.wnc=gps_epoch.GPSweek();
    gps_nav.sat_id=ephemeris.getPRNID();
    gps_nav.wn=gps_epoch.GPSModWeek();
    gps_nav.ca_or_pon_l2=ephemeris.getCodeFlags();
    gps_nav.ura=ephemeris.getAccFlag();
    gps_nav.health=ephemeris.getHealth();
    gps_nav.l2_data_flag=ephemeris.getL2Pdata();
    gps_nav.iodc=ephemeris.getIODC();	
    gps_nav.iode2=ephemeris.getIODE();
    gps_nav.iode3=ephemeris.getIODE();
    gps_nav.fit_int_flag=ephemeris.getFitInt();
    gps_nav.t_gd=ephemeris.getTgd();
    gps_nav.t_oc=ephemeris.getToc();
    gps_nav.a_f2=ephemeris.getAf2();
    gps_nav.a_f1=ephemeris.getAf1();
    gps_nav.a_f0=ephemeris.getAf0();
    gps_nav.c_rs=ephemeris.getCrs();
    gps_nav.delta_n=ephemeris.getDn();
    gps_nav.m_0=ephemeris.getM0();
    gps_nav.c_uc=ephemeris.getCuc();
    gps_nav.e=ephemeris.getEcc();
    gps_nav.c_us=ephemeris.getCus();
    gps_nav.sqrt_a=ephemeris.getAhalf();
    gps_nav.t_oe=ephemeris.getToe();
    gps_nav.c_ic=ephemeris.getCic();
    gps_nav.omega_0=ephemeris.getOmega0();
    gps_nav.c_is=ephemeris.getCis();
    gps_nav.i_0=ephemeris.getI0();
    gps_nav.c_rc=ephemeris.getCrc();
    gps_nav.omega=ephemeris.getW();
    gps_nav.omega_dot=ephemeris.getOmegaDot();
    gps_nav.i_dot=ephemeris.getIDot();
    gps_nav.wnt_oc=gps_epoch.GPSModWeek();
    gps_nav.wnt_oe=gps_epoch.GPSModWeek();
  }catch(gpstk::InvalidParameter &e){
    /* ignore the Error for including subframes 4 and 5 */
    //std::cout << "generate GPS NAV" << e.what() << std::endl;
  }catch(gpstk::InvalidRequest &e){
    /* this should not happen */
    //std::cout << "generate GPS NAV" << e.what() << std::endl;
  }catch(...){
    //std::cout << "generate GPS NAV Exception" << std::endl;
  }
}

unsigned int CAsteRx1Process::get_subframe_id(TGPSFrame &frame)
{
  unsigned long int SFword2=(unsigned long int)frame.frame_bits[1];
  SFword2&=0x00000700;// Strip all but three bit subframe ID
  SFword2>>=8;// Right shift to move to lsbs;

  return SFword2;
}

void CAsteRx1Process::add_subframe(TGPSFrame &frame)
{
  TGPSFrames gps_frames;
  unsigned int i=0,j=0;
  TGPSNav gps_nav;

  try{
    if(this->get_subframe_id(frame)<4)
    {
      this->ephemeris_data[frame.sat_id].ephemeris.addSubframe(frame.frame_bits,frame.wn,frame.sat_id,frame.track_id);
      if(!this->ephemeris_data[frame.sat_id].sync)
      {
        if(this->get_subframe_id(frame)==1)// the first frame has been received for the first time
        {
          this->ephemeris_data[frame.sat_id].sync=true;
          this->ephemeris_data[frame.sat_id].missing_frames[this->get_subframe_id(frame)-1]=false;
        }
      }
      else
      {
        this->ephemeris_data[frame.sat_id].missing_frames[this->get_subframe_id(frame)-1]=false;
        if(!this->ephemeris_data[frame.sat_id].missing_frames[0] && !this->ephemeris_data[frame.sat_id].missing_frames[1] &&
           !this->ephemeris_data[frame.sat_id].missing_frames[2])
        {
          this->ephemeris_data[frame.sat_id].missing_frames[0]=true;
          this->ephemeris_data[frame.sat_id].missing_frames[1]=true;
          this->ephemeris_data[frame.sat_id].missing_frames[2]=true;
          // process the data and build a GPSNav structure
          this->generate_gps_nav(this->ephemeris_data[frame.sat_id].ephemeris,gps_nav);
          gps_nav.local_timestamp=frame.local_timestamp;
          gps_frames.wn=this->ephemeris_data[frame.sat_id].ephemeris.weeknum;
          gps_frames.sat_id=this->ephemeris_data[frame.sat_id].ephemeris.PRNID;
          gps_frames.track_id=this->ephemeris_data[frame.sat_id].ephemeris.tracker;
          for(i=0;i<3;i++)
          {
            for(j=0;j<10;j++)
              gps_frames.subframes[i][j]=this->ephemeris_data[frame.sat_id].ephemeris.subframeStore[i][j];
          }
          this->gps_nav_data.push(gps_nav);
          this->gps_raw_data.push(gps_frames);
        }
      }
    }
  }catch(gpstk::InvalidParameter &e){
    /* ignore the Error for including subframes 4 and 5 */
    this->ephemeris_data[frame.sat_id].missing_frames[0]=true;
    this->ephemeris_data[frame.sat_id].missing_frames[1]=true;
    this->ephemeris_data[frame.sat_id].missing_frames[2]=true;
  }catch(gpstk::InvalidRequest &e){
    /* this should not happen */
    this->ephemeris_data[frame.sat_id].sync=false;
    this->ephemeris_data[frame.sat_id].missing_frames[0]=true;
    this->ephemeris_data[frame.sat_id].missing_frames[1]=true;
    this->ephemeris_data[frame.sat_id].missing_frames[2]=true;
  }catch(...){
    //std::cout << "add subframe Exception" << std::endl;
  }
}

bool CAsteRx1Process::is_new_gps_nav_data_available(void)
{
  if(this->gps_nav_data.size()>0)
    return true;
  else
    return false;
}

void CAsteRx1Process::get_gps_nav_data(TGPSNav &gps_nav,TGPSFrames &frames)
{
  if(this->gps_nav_data.size()>0)
  {
    gps_nav=this->gps_nav_data.front();
    this->gps_nav_data.pop();
    frames=this->gps_raw_data.front();
    this->gps_raw_data.pop();
  }
}

CAsteRx1Process::~CAsteRx1Process()
{
  this->ephemeris_data.clear();
}

