#include "stream_gps.h"
#include <iomanip>

std::ostream& operator<< (std::ostream& out, TMeasEpoch &meas_epoch)
{
  unsigned int i,j;

  out << "Common flags: " << std::endl;
  if(meas_epoch.multipath_mitigation)
    out << "  Multipath mitigation enabled" << std::endl;
  else
    out << "  Multipath mitigation disabled" << std::endl;
  if(meas_epoch.smoothing_of_code)
    out << "  At least one of the code measurements is smoothed" << std::endl;
  else
    out << "  None of the code measurements are smoothed" << std::endl;
  for(i=0;i<meas_epoch.type1_data.size();i++)
  {
    out << "Sattelite " << meas_epoch.type1_data[i].sat_id << " on channel " << meas_epoch.type1_data[i].rx_channel << std::endl;
    out << "  Master signal: " << signal_type_name[meas_epoch.type1_data[i].signal_type] << std::endl;
    out << "  Antenna: " << (int)meas_epoch.type1_data[i].antenna_id << std::endl;
    out << "  Pseudo range: " << meas_epoch.type1_data[i].pseudo_range << " m" << std::endl;
    out << "  Doppler: " << meas_epoch.type1_data[i].doppler << " Hz" << std::endl;
    out << "  Carrier phase: " << meas_epoch.type1_data[i].carrier_phase << " cycles" << std::endl;
    out << "  Carrier/noise ratio: " << meas_epoch.type1_data[i].cn0 << " dB-Hz" << std::endl;
    out << "  Lock time: " << meas_epoch.type1_data[i].lock_time << " s" << std::endl;
    if(meas_epoch.type1_data[i].pseudorange_smoothed)
      out << "  Pseudo ranfe measurement is smoothed" << std::endl;
    else
      out << "  Pseudo ranfe measurement is not smoothed" << std::endl;
    if(meas_epoch.type1_data[i].smoothing_int_reached)
      out << "  The smoothing filter has reached the requested smoothing interval" << std::endl;
    else
      out << "  The smoothing filter has not reached the requested smoothing interval" << std::endl;
    if(meas_epoch.type1_data[i].half_cycle_ambiguity)
      out << "  Carrier phase has a half-cycle ambiguity" << std::endl;
    else
      out << "  Carrier phase does not have a half-cycle ambiguity" << std::endl;
    if(meas_epoch.type1_data[i].type2_data.size()>0)
    {
      for(j=0;j<meas_epoch.type1_data[i].type2_data.size();j++)
      {
        out << "  Inter-channel differences" << std::endl;
        out << "    Master signal: " << signal_type_name[meas_epoch.type1_data[i].type2_data[j].signal_type] << std::endl;
        out << "    Antenna: " << (int)meas_epoch.type1_data[i].type2_data[j].antenna_id << std::endl;
        out << "    Pseudo range: " << meas_epoch.type1_data[i].type2_data[j].pseudo_range << " m" << std::endl;
        out << "    Doppler: " << meas_epoch.type1_data[i].type2_data[j].doppler << " Hz" << std::endl;
        out << "    Carrier phase: " << meas_epoch.type1_data[i].type2_data[j].carrier_phase << " cycles" << std::endl;
        out << "    Carrier/noise ratio: " << meas_epoch.type1_data[i].type2_data[j].cn0 << " dB-Hz" << std::endl;
        out << "    Lock time: " << meas_epoch.type1_data[i].type2_data[j].lock_time << " s" << std::endl;
        if(meas_epoch.type1_data[i].type2_data[j].pseudorange_smoothed)
          out << "  Pseudo ranfe measurement is smoothed" << std::endl;
        else
          out << "  Pseudo ranfe measurement is not smoothed" << std::endl;
        if(meas_epoch.type1_data[i].type2_data[j].smoothing_int_reached)
          out << "  The smoothing filter has reached the requested smoothing interval" << std::endl;
        else
          out << "  The smoothing filter has not reached the requested smoothing interval" << std::endl;
        if(meas_epoch.type1_data[i].type2_data[j].half_cycle_ambiguity)
          out << "  Carrier phase has a half-cycle ambiguity" << std::endl;
        else
          out << "  Carrier phase does not have a half-cycle ambiguity" << std::endl;
      }
    }
  }
 
  return out;
}

std::ostream& operator<< (std::ostream& out, TMeasExtra &meas_extra)
{
  unsigned int i;

  out << "Doppler variance factor: " <<  meas_extra.doppler_var_factor << " Hz2/cycles2" << std::endl;
  for(i=0;i<meas_extra.blocks.size();i++)
  {
    out << "Block:" << std::endl;
    out << "  Receiver channel: " << meas_extra.blocks[i].rx_channel << std::endl;
    out << "  Master signal: " << signal_type_name[meas_extra.blocks[i].signal_type] << std::endl;
    out << "  Antenna: " << meas_extra.blocks[i].antenna_id << std::endl;
    out << "  Multipath correction: " << meas_extra.blocks[i].mp_correction << " m" << std::endl;
    out << "  Smoothing correction: " << meas_extra.blocks[i].smoothing_correction << " m" << std::endl;
    out << "  Code tracking noise variance: " << meas_extra.blocks[i].code_var << " m2" << std::endl;
    out << "  Carrier tracking noise variance: " << meas_extra.blocks[i].carrier_var << " mcycle2" << std::endl;
    out << "  Lock time: " << meas_extra.blocks[i].lock_time << " s" << std::endl;
  }

  return out;
}

std::ostream& operator<< (std::ostream& out, TGPSNav &gps_nav)
{
  out << "GPSNav block for satellite " << gps_nav.sat_id << std::endl;
  out << "  Time of week: " << gps_nav.gps_timestamp.tow << std::endl;
  out << "  Week number: " << gps_nav.gps_timestamp.wnc << std::endl;
  out << "  Code on L2 channel: " << gps_nav.ca_or_pon_l2 << std::endl;
  out << "  User range accuracy index: " << gps_nav.ura << std::endl;
  out << "  Health: " << gps_nav.health << std::endl;
  out << "  Data flag for L2 p-code: " << gps_nav.l2_data_flag << std::endl;
  out << "  Issue of data, clock: " << gps_nav.iodc << std::endl;
  out << "  Issue of data, ephemeris (subframe 2): " << gps_nav.iode2 << std::endl;
  out << "  Issue of data, ephemeris (subframe 3): " << gps_nav.iode3 << std::endl;
  out << "  Curve fit interval: " << gps_nav.fit_int_flag << std::endl;
  out << "  Estimated group delay differential (Tgd): " << gps_nav.t_gd << " s" << std::endl;
  out << "  Clock data reference time (Toc): " << gps_nav.t_oc << " s" << std::endl;
  out << "  SV clock aging: " << gps_nav.a_f2 << " s/s2" << std::endl;
  out << "  SV clock drift: " << gps_nav.a_f1 << " s/s" << std::endl;
  out << "  SV clock bias: " << gps_nav.a_f0 << " s" << std::endl;
  out << "  Amplitude of the sine harmonic correction term to the orbit radius (Crs): " << gps_nav.c_rs << " m" << std::endl;
  out << "  Mean motion difference: " << gps_nav.delta_n << " semi-circle/s" << std::endl;
  out << "  Mean anomaly at reference time: " << gps_nav.m_0 << " semi-circle" << std::endl;
  out << "  Amplitude of the cosine harmonic correction term to the argument of latitude (Cuc): " << gps_nav.c_uc << " rad" << std::endl;
  out << "  Eccentricity: " << gps_nav.e << std::endl;
  out << "  Amplitude of the cosine harmonic correction term to the argument of latitude (Cus): " << gps_nav.c_us << " rad" << std::endl;
  out << "  Square root of the semi-major axis: " << gps_nav.sqrt_a << " m1/2" << std::endl;
  out << "  Reference time ephemeris (Toe): " << gps_nav.t_oe << " s" << std::endl;
  out << "  Amplitude of the cosine harmonic correction term to the angle of inclination (Cic): " << gps_nav.c_ic << " rad" << std::endl;
  out << "  Longitude of ascending node of orbit plane at weekly epoch: " << gps_nav.omega_0 << " semi-circle" << std::endl;
  out << "  Amplitude of the sine harmonic correction term to the angle of inclination (Cis): " << gps_nav.c_is << " rad" << std::endl;
  out << "  Inclination angle at reference time: " << gps_nav.i_0 << " semi-circle" << std::endl;
  out << "  Amplitude of the cosine harmonic correction term to the orbit radius (Crc): " << gps_nav.c_rc << " m" << std::endl;
  out << "  Argument of peregee: " << gps_nav.omega << " semi-circle" << std::endl;
  out << "  Rate of right ascension: " << gps_nav.omega_dot << " semi-circle/s" << std::endl;
  out << "  Rate of inclination angle: " << gps_nav.i_dot << " semi-circle/s" << std::endl;

  return out;
}

std::ostream& operator<< (std::ostream& out, TGPSAlm &gps_alm)
{
  out << "GPSAlm block for satellite: " << gps_alm.sat_id << std::endl;
  out << "Eccentricity: " << gps_alm.e << std::endl;
  out << "Almanac reference time of week (Toa): " << gps_alm.t_oa << " s" << std::endl;
  out << "Inclinationa ngle at reference time: " << gps_alm.delta_i << " semi-circle" << std::endl;
  out << "Rate of right ascension: " << gps_alm.omega_dot << " semi-circle/s" << std::endl;
  out << "Square root of the semi-major axis: " << gps_alm.sqrt_a << " m1/2" << std::endl;
  out << "Longitude of ascending node of orbit plane at weekly epoch: " << gps_alm.omega_0 << " semi-circle" << std::endl;
  out << "Argument of peregee: " << gps_alm.omega << " semi-circle" << std::endl;
  out << "Mean anomaly at reference time: " << gps_alm.m_0 << " semi-circle" << std::endl;
  out << "SV clock drift: " << gps_alm.a_f1 << " s/s" << std::endl;
  out << "SV clock bias: " << gps_alm.a_f0 << " s" <<  std::endl;

  return out;
}

std::ostream& operator<< (std::ostream& out, TGPSIon &gps_ion)
{
  out << "GPSIon block for satellite: " << gps_ion.sat_id << std::endl;
  out << "Vertical delay coefficient 0: " << gps_ion.alpha_0 << " s" << std::endl;
  out << "Vertical delay coefficient 1: " << gps_ion.alpha_1 << " s/semi-circle" << std::endl;
  out << "Vertical delay coefficient 2: " << gps_ion.alpha_2 << " s/semi-circle2" << std::endl;
  out << "Vertical delay coefficient 3: " << gps_ion.alpha_3 << " s/semi-circle3" << std::endl;
  out << "Model period coefficient 0: " << gps_ion.beta_0 << " s" << std::endl;
  out << "Model period coefficient 1: " << gps_ion.beta_1 << " s/semi-circle" << std::endl;
  out << "Model period coefficient 2: " << gps_ion.beta_2 << " s/semi-circle2" << std::endl;
  out << "Model period coefficient 3: " << gps_ion.beta_3 << " s/semi-circle3" << std::endl;

  return out;
}

std::ostream& operator<< (std::ostream& out, TGPSUtc &gps_utc)
{
  out << "GPSUtc block for satellite: " << gps_utc.sat_id << std::endl;
  out << "First order term of polynomial: " << gps_utc.a_1 << " s/s" << std::endl;
  out << "constant term of polynomial: " << gps_utc.a_0 << " s" << std::endl;
  out << "Reference time for UTC data (Tot): " << gps_utc.t_ot << " s" << std::endl;
  out << "Delta time due to leap seconds when effectivity time is not in the past: " << gps_utc.del_t_ls << " s" << std::endl;
  out << "Effectivity time of leap seconds (week): " << gps_utc.wn_lsf << " weelk" << std::endl;
  out << "Effectivity time of leap seconds (day): " << gps_utc.dn << " day" << std::endl;
  out << "Delta time due to leap seconds when effectivity time is not in the past: " << gps_utc.del_t_lsf << " s" << std::endl;

  return out;
}

std::ostream& operator<< (std::ostream& out, TGPSFrame &gps_frame)
{
  unsigned int i=0;

  out << "GPS Raw Frame for satellite: " << gps_frame.sat_id << std::endl;
  out << "  Week number: " << gps_frame.wn << std::endl;
  out << "  Tracking channel: " << gps_frame.track_id << std::endl;
  out << "  Frame data: " << std::endl;
  for(i=0;i<10;i++)
  {
    out << "0x" << std::hex << gps_frame.frame_bits[i];
    if(i<9)
      out << ",";
  }
  out << std::dec << std::endl;

  return out;
}

std::ostream& operator<< (std::ostream& out, TPVTCartesian &pvt_cartesian)
{
  out << "PVT mode: ";
  switch(pvt_cartesian.mode)
  {
    case no_pvt: out << "No PVT avaialbe" << std::endl;
                 break;
    case stand_alone_pvt: out << "stand alone" << std::endl;
                          break;
    case diff_pvt: out << "Differential" << std::endl;
                   break; 
    case fixed_loc: out << "Fixed location" << std::endl;
                    break;
    case rtk_fixed_amb: out << "RTK with fixed ambiguities" << std::endl;
                        break;
    case rtk_float_amb: out << "RTK with float a,biguities" << std::endl;
                        break;
    case sbas_pvt: out << "SBAS aided" << std::endl;
                   break;
    default: out << std::endl;
             break;
  }
  if(pvt_cartesian.solution==pvt_3d)
    out << "3D PVT Solution" << std::endl;
  else
    out << "2D PVT Solution" << std::endl;
  out << "Error: ";
  switch(pvt_cartesian.error)
  { 
    case no_error: out << "none" << std::endl;
                   break;
    case not_enough_meas: out << "Not enough measurements" << std::endl;
                          break;
    case not_enough_eph: out << "Not enough ephemerides available" << std::endl;
                         break;
    case dop_too_large: out << "DOP too large" << std::endl;
                        break;
    case res_too_large: out << "Sum of squared residuals too large" << std::endl;
                        break;
    case no_conv: out << "No convergence" << std::endl;
                  break;
    case not_enough_meas_outliers: out << "Not enough measurements after outlier rejection" << std::endl;
                                   break;
    case export_laws: out << "Position output prohibited due to export laws" << std::endl;
                      break;
    case not_enough_diff_corr: out << "Not enough differential correlation available" << std::endl;
                               break;
    case no_base_coord: out << "Base station coordinates unavailable" << std::endl;
                        break;
    default: out << std::endl;
             break;
  }
  out << "Position: " << std::endl;
  out << "  x: " << pvt_cartesian.x << " m" << std::endl;
  out << "  y: " << pvt_cartesian.y << " m" << std::endl;
  out << "  z: " << pvt_cartesian.z << " m" << std::endl;
  out << "Velocity: " << std::endl;
  out << "  x: " << pvt_cartesian.vx << " m/s" << std::endl;
  out << "  y: " << pvt_cartesian.vy << " m/s" << std::endl;
  out << "  z: " << pvt_cartesian.vz << " m/s" << std::endl;
  out << "Undulation: " << pvt_cartesian.undulation << " m" << std::endl;
  out << "Course over ground: " << pvt_cartesian.cog << " degrees" << std::endl;
  out << "Receiver clock bias: " << pvt_cartesian.rx_clk_bias << " ms" << std::endl;
  out << "Receiver clock drift: " << pvt_cartesian.rx_clk_drift << " ppm" << std::endl;
  if(pvt_cartesian.time_system==gps_time) 
    out << "Using GPS time" << std::endl;
  else
    out << "Using Galileo time" << std::endl;
  out << "Datum: ";
  switch(pvt_cartesian.datum)
  {
    case wgs_84: out << "WGS84" << std::endl;
                 break;
    case galileo: out << "Galileo" << std::endl;
                  break;
    case pz_90_02: out << "PZ-90.02" << std::endl;
                   break;
    case user_datum1: out << "User datum 1" << std::endl;
                      break;
    case user_datum2: out << "User datum 2" << std::endl;
                      break;
    default: out << std::endl;
             break;
  }
  out << "Used " << pvt_cartesian.num_sat << " satellites" << std::endl;

  return out;
}

std::ostream& operator<< (std::ostream& out, TPVTGeodetic &pvt_geodetic)
{
  out << "PVT mode: ";
  switch(pvt_geodetic.mode)
  {
    case no_pvt: out << "No PVT avaialbe" << std::endl;
                 break;
    case stand_alone_pvt: out << "stand alone" << std::endl;
                          break;
    case diff_pvt: out << "Differential" << std::endl;
                   break;
    case fixed_loc: out << "Fixed location" << std::endl;
                    break;
    case rtk_fixed_amb: out << "RTK with fixed ambiguities" << std::endl;
                        break;
    case rtk_float_amb: out << "RTK with float a,biguities" << std::endl;
                        break;
    case sbas_pvt: out << "SBAS aided" << std::endl;
                   break;
    default: out << std::endl;
             break;
  }
  if(pvt_geodetic.solution==pvt_3d)
    out << "3D PVT Solution" << std::endl;
  else
    out << "2D PVT Solution" << std::endl;
  out << "Error: ";
  switch(pvt_geodetic.error)
  {
    case no_error: out << "none" << std::endl;
                   break;
    case not_enough_meas: out << "Not enough measurements" << std::endl;
                          break;
    case not_enough_eph: out << "Not enough ephemerides available" << std::endl;
                         break;
    case dop_too_large: out << "DOP too large" << std::endl;
                        break;
    case res_too_large: out << "Sum of squared residuals too large" << std::endl;
                        break;
    case no_conv: out << "No convergence" << std::endl;
                  break;
    case not_enough_meas_outliers: out << "Not enough measurements after outlier rejection" << std::endl;
                                   break;
    case export_laws: out << "Position output prohibited due to export laws" << std::endl;
                      break;
    case not_enough_diff_corr: out << "Not enough differential correlation available" << std::endl;
                               break;
    case no_base_coord: out << "Base station coordinates unavailable" << std::endl;
                        break;
    default: out << std::endl;
             break;
  }
  out << "Position:" << std::endl;
  out << "  Latitude: " << pvt_geodetic.latitude << " rad" << std::endl;
  out << "  Longitude: " << pvt_geodetic.longitude << " rad" << std::endl;
  out << "  Height: " << pvt_geodetic.h << " m" << std::endl;
  out << "Velocity: " << std::endl;
  out << "  North: " << pvt_geodetic.vn << " m/s" << std::endl;
  out << "  East: " << pvt_geodetic.ve << " m/s" << std::endl;
  out << "  Up: " << pvt_geodetic.vu << " m/s" << std::endl;
  out << "Undulation: " << pvt_geodetic.undulation << " m" << std::endl;
  out << "Course over ground: " << pvt_geodetic.cog << " degrees" << std::endl;
  out << "Receiver clock bias: " << pvt_geodetic.rx_clk_bias << " ms" << std::endl;
  out << "Receiver clock drift: " << pvt_geodetic.rx_clk_drift << " ppm" << std::endl;
  if(pvt_geodetic.time_system==gps_time)
    out << "Using GPS time" << std::endl;
  else
    out << "Using Galileo time" << std::endl;
  out << "Datum: ";
  switch(pvt_geodetic.datum)
  {
    case wgs_84: out << "WGS84" << std::endl;
                 break;
    case galileo: out << "Galileo" << std::endl;
                  break;
    case pz_90_02: out << "PZ-90.02" << std::endl;
                   break;
    case user_datum1: out << "User datum 1" << std::endl;
                      break;
    case user_datum2: out << "User datum 2" << std::endl;
                      break;
    default: out << std::endl;
             break;
  }
  out << "Used " << pvt_geodetic.num_sat << " satellites" << std::endl;

  return out;
}

std::ostream& operator<< (std::ostream& out, TPVTCov &pvt_cov)
{
  out << "Covariance:" << std::endl;
  out << std::fixed << std::setw(11) << std::setprecision(6) << "/ " << pvt_cov.cov_00 << " " << pvt_cov.cov_01 << " " << pvt_cov.cov_02 << " " << pvt_cov.cov_03 << " \\" << std::endl;
  out << std::fixed << std::setw(11) << std::setprecision(6) << "| " << pvt_cov.cov_01 << " " << pvt_cov.cov_11 << " " << pvt_cov.cov_12 << " " << pvt_cov.cov_13 << " |" << std::endl;
  out << std::fixed << std::setw(11) << std::setprecision(6) << "| " << pvt_cov.cov_02 << " " << pvt_cov.cov_12 << " " << pvt_cov.cov_22 << " " << pvt_cov.cov_23 << " |" << std::endl;
  out << std::fixed << std::setw(11) << std::setprecision(6) << "\\ " << pvt_cov.cov_03 << " " << pvt_cov.cov_13 << " " << pvt_cov.cov_23 << " " << pvt_cov.cov_33 << " /" << std::endl;

  return out;
}

std::ostream& operator<< (std::ostream& out, TPVTDOP &pvt_dop)
{
  out << "Used " << pvt_dop.num_sat << " satellites" << std::endl;
  out << "Position dilution of precision: " << pvt_dop.p_dop << std::endl; 
  out << "time dilution of precision: " << pvt_dop.t_dop << std::endl; 
  out << "Horitzontal dilution of precision: " << pvt_dop.h_dop << std::endl; 
  out << "Vertical dilution of precision: " << pvt_dop.v_dop << std::endl; 
  out << "Horitzontal protection level: " << pvt_dop.hpl << std::endl; 
  out << "Vertical protection level: " << pvt_dop.vpl << std::endl; 

  return out;
}
// ChannelStatus
/*  std::cout << "gps timestamp: " << this->channel_status.gps_timestamp << std::endl;
  std::cout << "num channles: " << this->channel_status.info.size() << std::endl;
  for(i=0;i<this->channel_status.info.size();i++)
  {
    std::cout << "  Sat ID: " << (int)this->channel_status.info[i].sat_id << std::endl;
    std::cout << "  Azimuth: " << this->channel_status.info[i].azimuth << std::endl;
    std::cout << "  rise/set: " << this->channel_status.info[i].rise_set << std::endl;
    std::cout << "  Health: " << this->channel_status.info[i].health << std::endl;
    std::cout << "  Elevation: " << this->channel_status.info[i].elevation << std::endl;
    for(j=0;j<this->channel_status.info[i].state.size();j++)
    {
      std::cout << "   Antenna ID: " << (int)this->channel_status.info[i].state[j].antenna_id << std::endl;
      std::cout << "   Tracking status: " << this->channel_status.info[i].state[j].track_status << std::endl;
      std::cout << "   PVT status: " << this->channel_status.info[i].state[j].pvt_status << std::endl;
    }
  }*/

//ReceiverStatus
/*  std::cout << "gps timestamp: " << this->receiver_status.gps_timestamp << std::endl;
  std::cout << "CPU load: " << this->receiver_status.cpu_load << std::endl;
  std::cout << "up time: " << this->receiver_status.up_time << std::endl;
  for(i=0;i<this->receiver_status.agc.size();i++)
  {
    std::cout << "  Frontend: " << this->receiver_status.agc[i].frontend_id << std::endl;;
    std::cout << "  Antenna: " << this->receiver_status.agc[i].antenna_id << std::endl;;
    std::cout << "  Gain: " << this->receiver_status.agc[i].gain << std::endl;;
    std::cout << "  Sample var: " << this->receiver_status.agc[i].sample_var << std::endl;;
    std::cout << "  Blanking stat: " << this->receiver_status.agc[i].blanking_stat << std::endl;;
  }*/

// SatVisibility
/*  std::cout << "Num sat: " << this->sat_info.size() << std::endl;
  for(i=0;i<this->sat_info.size();i++)
  {
    std::cout << "Sat ID: " << this->sat_info[i].sat_id << std::endl;
    std::cout << "Azmimuth: " << this->sat_info[i].azimuth << std::endl;
    std::cout << "Elevation: " << this->sat_info[i].elevation << std::endl;
    std::cout << "rise/set: " << this->sat_info[i].rise_set << std::endl;
    std::cout << "Sat info: " << this->sat_info[i].sat_info << std::endl;
  }*/

